@@ -41,13 +41,13 @@
  *****************************************************************************/
 
 /* Program version and copyright message */
-#define VERSION_MESSAGE     "vlc 0.2.61 Urumov " \
+#define VERSION_MESSAGE     "vlc 0.2.62 Urumov " \
                           /* "(" PROGRAM_BUILD ") (" PROGRAM_OPTIONS ")\n" */ \
                             "Copyright 1996-2000 VideoLAN\n"
-#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.61" \
+#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.62" \
                             " Urumov - (c)1996-2000 VideoLAN\n"
 
-#define VERSION             "0.2.61"
+#define VERSION             "0.2.62"
 
 /*****************************************************************************
  * General compilation options
@@ -119,8 +119,8 @@
  * Paths
  */
 
-#define DATA_PATH                       "/usr/local/share/videolan"
-#define PLUGIN_PATH                     "/usr/local/lib/videolan/vlc"
+#define DATA_PATH                       "/usr/share/videolan"
+#define PLUGIN_PATH                     "/usr/lib/videolan/vlc"
 
 #define MAX_PLUGIN_COUNT                32
 
@@ -369,13 +369,8 @@
 #define VOUT_FB_DEV_VAR                 "vlc_fb_dev"
 #define VOUT_FB_DEV_DEFAULT             "/dev/fb0"
 
-/* Some frame buffers aren't able to support double buffering.
- * We don't want to lose one frame out of 2, so we may set the
- * FB_NOYPAN
- */
-// #define FB_NOYPAN
-
-#define VOUT_TITLE                      "VideoLAN Client 0.2.61"
+/* The default video output window title */
+#define VOUT_TITLE                      "VideoLAN Client 0.2.62"
 
 /*****************************************************************************
  * Video parser configuration
