@@ -2,7 +2,7 @@
  * video_parser.c : video parser thread
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: video_parser.c,v 1.74 2001/02/13 13:01:14 massiot Exp $
+ * $Id: video_parser.c,v 1.76 2001/02/20 15:03:00 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
@@ -287,20 +287,19 @@ static int InitThread( vpar_thread_t *p_vpar )
     p_vpar->pp_vdec[0]->b_error = 0;
     p_vpar->pp_vdec[0]->p_vpar = p_vpar;
 
-#   if VDEC_NICE
+#   ifndef SYS_BEOS
+#       if VDEC_NICE
     /* Re-nice ourself */
     if( nice(VDEC_NICE) == -1 )
     {
         intf_WarnMsg( 2, "vpar warning : couldn't nice() (%s)",
                       strerror(errno) );
     }
+#       endif
 #   endif
 #endif
 
     /* Initialize lookup tables */
-#if defined(MPEG2_COMPLIANT) && !defined(VDEC_DFT)
-    vpar_InitCrop( p_vpar );
-#endif
     vpar_InitMbAddrInc( p_vpar );
     vpar_InitDCTTables( p_vpar );
     vpar_InitPMBType( p_vpar );
